"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Builder = Builder;
exports.any = void 0;
exports.array = array;
exports.defaults = void 0;
exports.intersect = intersect;
exports.object = object;
exports.record = record;
exports.tuple = tuple;
exports.union = union;
exports.withPartialKeysOf = withPartialKeysOf;
var _vitest = /*#__PURE__*/require("@fast-check/vitest");
const identifier = /*#__PURE__*/_vitest.fc.stringMatching(/*#__PURE__*/new RegExp('^[$_a-zA-Z][$_a-zA-Z0-9]*$', 'u'));
function withPartialKeysOf(model, keyCount) {
  return _vitest.fc.tuple(model, _vitest.fc.array(_vitest.fc.boolean(), {
    minLength: keyCount,
    maxLength: keyCount
  })).map(([object, indices]) => {
    const keys = Object.keys(object);
    const out = Array.of();
    for (let ix = 0, len = keys.length; ix < len; ix++) if (indices[ix]) out.push(keys[ix]);
    return [object, out];
  });
}
const defaultObjectConstraints = {
  keyCount: 10
};
const defaults = exports.defaults = {
  object: defaultObjectConstraints
};
const boolean = /*#__PURE__*/_vitest.fc.record({
  type: /*#__PURE__*/_vitest.fc.constant('boolean')
});
const integer = /*#__PURE__*/_vitest.fc.record({
  type: /*#__PURE__*/_vitest.fc.constant('integer')
});
const number = /*#__PURE__*/_vitest.fc.record({
  type: /*#__PURE__*/_vitest.fc.constant('number')
});
const string = /*#__PURE__*/_vitest.fc.record({
  type: /*#__PURE__*/_vitest.fc.constant('string')
});
function tuple(cont) {
  return _vitest.fc.record({
    type: _vitest.fc.constant('array'),
    items: _vitest.fc.array(cont('tree')),
    additionalItems: _vitest.fc.constant(false)
  }).map(node => ({
    ...node,
    minItems: node.items.length,
    maxItems: node.items.length
  }));
}
function object(cont, {
  keyCount,
  ...constraints
} = defaults.object) {
  return _vitest.fc.record({
    type: _vitest.fc.constant('object'),
    properties: withPartialKeysOf(_vitest.fc.dictionary(identifier, cont('tree'), constraints), keyCount ?? defaults.object.keyCount)
  }).map(({
    properties: [properties, required],
    ...node
  }) => ({
    ...node,
    properties,
    required
  }));
}
function array(cont) {
  return _vitest.fc.record({
    type: _vitest.fc.constant('array'),
    items: cont('tree')
  });
}
function record(cont) {
  return _vitest.fc.record({
    type: _vitest.fc.constant('object'),
    additionalProperties: cont('tree')
  });
}
function union(cont) {
  return _vitest.fc.record({
    anyOf: _vitest.fc.array(cont('tree'))
  });
}
function intersect(cont) {
  return _vitest.fc.record({
    allOf: _vitest.fc.array(cont('tree'))
  });
}
function Builder(constraints = defaults, arbitraries) {
  return cont => ({
    boolean,
    integer,
    number,
    string,
    array: array(cont),
    record: record(cont),
    tuple: tuple(cont),
    union: union(cont),
    intersect: intersect(cont),
    object: object(cont, constraints.object),
    ...arbitraries,
    tree: _vitest.fc.oneof(cont('boolean'), cont('integer'), cont('number'), cont('string'), cont('array'), cont('record'), cont('tuple'), cont('object'))
  });
}
const any = constraints => _vitest.fc.letrec(Builder(constraints)).tree;
exports.any = any;
//# sourceMappingURL=arbitrary.js.map