import { fc } from '@fast-check/vitest';
const identifier = /*#__PURE__*/fc.stringMatching(/*#__PURE__*/new RegExp('^[$_a-zA-Z][$_a-zA-Z0-9]*$', 'u'));
export function withPartialKeysOf(model, keyCount) {
  return fc.tuple(model, fc.array(fc.boolean(), {
    minLength: keyCount,
    maxLength: keyCount
  })).map(([object, indices]) => {
    const keys = Object.keys(object);
    const out = Array.of();
    for (let ix = 0, len = keys.length; ix < len; ix++) if (indices[ix]) out.push(keys[ix]);
    return [object, out];
  });
}
const defaultObjectConstraints = {
  keyCount: 10
};
export const defaults = {
  object: defaultObjectConstraints
};
const boolean = /*#__PURE__*/fc.record({
  type: /*#__PURE__*/fc.constant('boolean')
});
const integer = /*#__PURE__*/fc.record({
  type: /*#__PURE__*/fc.constant('integer')
});
const number = /*#__PURE__*/fc.record({
  type: /*#__PURE__*/fc.constant('number')
});
const string = /*#__PURE__*/fc.record({
  type: /*#__PURE__*/fc.constant('string')
});
export function tuple(cont) {
  return fc.record({
    type: fc.constant('array'),
    items: fc.array(cont('tree')),
    additionalItems: fc.constant(false)
  }).map(node => ({
    ...node,
    minItems: node.items.length,
    maxItems: node.items.length
  }));
}
export function object(cont, {
  keyCount,
  ...constraints
} = defaults.object) {
  return fc.record({
    type: fc.constant('object'),
    properties: withPartialKeysOf(fc.dictionary(identifier, cont('tree'), constraints), keyCount ?? defaults.object.keyCount)
  }).map(({
    properties: [properties, required],
    ...node
  }) => ({
    ...node,
    properties,
    required
  }));
}
export function array(cont) {
  return fc.record({
    type: fc.constant('array'),
    items: cont('tree')
  });
}
export function record(cont) {
  return fc.record({
    type: fc.constant('object'),
    additionalProperties: cont('tree')
  });
}
export function union(cont) {
  return fc.record({
    anyOf: fc.array(cont('tree'))
  });
}
export function intersect(cont) {
  return fc.record({
    allOf: fc.array(cont('tree'))
  });
}
export function Builder(constraints = defaults, arbitraries) {
  return cont => ({
    boolean,
    integer,
    number,
    string,
    array: array(cont),
    record: record(cont),
    tuple: tuple(cont),
    union: union(cont),
    intersect: intersect(cont),
    object: object(cont, constraints.object),
    ...arbitraries,
    tree: fc.oneof(cont('boolean'), cont('integer'), cont('number'), cont('string'), cont('array'), cont('record'), cont('tuple'), cont('object'))
  });
}
export const any = constraints => fc.letrec(Builder(constraints)).tree;
//# sourceMappingURL=arbitrary.js.map