import { t } from '@traversable/schema';
export const RAW = {
  any: {
    type: 'object',
    properties: {},
    nullable: true
  },
  null: {
    type: 'null',
    enum: [null]
  },
  boolean: {
    type: 'boolean'
  },
  integer: {
    type: 'integer'
  },
  number: {
    type: 'number'
  },
  string: {
    type: 'string'
  }
};
export { JsonSchema_any as any };
const JsonSchema_any = /*#__PURE__*/t.eq(RAW.any);
export { JsonSchema_null as null };
const JsonSchema_null = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq(RAW.null.type)
});
export { JsonSchema_boolean as boolean };
const JsonSchema_boolean = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq(RAW.boolean.type)
});
export const NumericBounds = /*#__PURE__*/t.object({
  minimum: /*#__PURE__*/t.optional(t.number),
  maximum: /*#__PURE__*/t.optional(t.number),
  exclusiveMinimum: /*#__PURE__*/t.optional(/*#__PURE__*/t.union(t.boolean, t.number)),
  exclusiveMaximum: /*#__PURE__*/t.optional(/*#__PURE__*/t.union(t.boolean, t.number))
});
export const SizeBounds = /*#__PURE__*/t.object({
  minLength: /*#__PURE__*/t.optional(t.number),
  maxLength: /*#__PURE__*/t.optional(t.number)
});
export { JsonSchema_integer as integer, JsonSchema_integer as IntegerSchema };
const JsonSchema_integer = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq(RAW.integer.type),
  ...NumericBounds.def
});
export { JsonSchema_number as number, JsonSchema_number as NumberSchema };
const JsonSchema_number = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq(RAW.number.type),
  ...NumericBounds.def
});
export { JsonSchema_string as string, JsonSchema_string as StringSchema };
const JsonSchema_string = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq(RAW.string.type),
  ...SizeBounds.def
});
export { JsonSchema_optional as optional };
const JsonSchema_optional = /*#__PURE__*/t.object({
  nullable: /*#__PURE__*/t.eq(true)
});
export { JsonSchema_const as const };
const JsonSchema_const = /*#__PURE__*/t.object({
  const: t.unknown
});
export { JsonSchema_enum as enum };
const JsonSchema_enum_ = /*#__PURE__*/t.object({
  enum: /*#__PURE__*/t.array(t.unknown, 'readonly')
});
const JsonSchema_enum = /*#__PURE__*/t.of(u => JsonSchema_enum_(u));
export { JsonSchema_array as array };
const JsonSchema_array = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('array'),
  items: t.unknown
});
export { JsonSchema_record as record };
const JsonSchema_record_ = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('object'),
  additionalProperties: t.unknown
});
const JsonSchema_record = u => JsonSchema_record_(u);
export { JsonSchema_union as union };
const JsonSchema_union_ = /*#__PURE__*/t.object({
  anyOf: /*#__PURE__*/t.array(t.unknown, 'readonly')
});
const JsonSchema_union = /*#__PURE__*/t.of(u => JsonSchema_union_(u));
export { JsonSchema_intersect as intersect };
const JsonSchema_intersect_ = /*#__PURE__*/t.object({
  allOf: /*#__PURE__*/t.array(t.unknown, 'readonly')
});
const JsonSchema_intersect = /*#__PURE__*/t.of(u => JsonSchema_intersect_(u));
export { JsonSchema_object as object };
const JsonSchema_object = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('object'),
  required: /*#__PURE__*/t.array(t.string),
  properties: t.unknown
});
export { JsonSchema_tuple as tuple };
const JsonSchema_tuple = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('array'),
  items: t.unknown,
  minItems: t.number,
  maxItems: t.number,
  additionalItems: /*#__PURE__*/t.eq(false)
});
const JsonSchema_nullary = /*#__PURE__*/t.union(JsonSchema_any, JsonSchema_null, JsonSchema_boolean, JsonSchema_integer, JsonSchema_number, JsonSchema_string);
const JsonSchema_special = /*#__PURE__*/t.union(JsonSchema_enum, JsonSchema_const);
const JsonSchema_unary = /*#__PURE__*/t.union(JsonSchema_array, JsonSchema_record, JsonSchema_union, JsonSchema_intersect, JsonSchema_tuple, JsonSchema_object);
export const is = {
  any: JsonSchema_any,
  null: JsonSchema_null,
  boolean: JsonSchema_boolean,
  integer: JsonSchema_integer,
  number: JsonSchema_number,
  string: JsonSchema_string,
  enum: JsonSchema_enum,
  const: JsonSchema_const,
  optional: JsonSchema_optional,
  array: JsonSchema_array,
  record: JsonSchema_record,
  union: JsonSchema_union,
  intersect: JsonSchema_intersect,
  tuple: JsonSchema_tuple,
  object: JsonSchema_object,
  nullary: JsonSchema_nullary,
  unary: JsonSchema_unary,
  special: JsonSchema_special
};
//# sourceMappingURL=specification.js.map